<?php
/* vim: set expandtab tabstop=4 softtabstop=4 shiftwidth=4:
  Codificación: UTF-8
  +----------------------------------------------------------------------+
  | Elastix version 0.5                                                  |f
  | http://www.elastix.org                                               |
  +----------------------------------------------------------------------+
  | Copyright (c) 2006 Palosanto Solutions S. A.                         |
  +----------------------------------------------------------------------+
  | Cdla. Nueva Kennedy Calle E 222 y 9na. Este                          |
  | Telfs. 2283-268, 2294-440, 2284-356                                  |
  | Guayaquil - Ecuador                                                  |
  | http://www.palosanto.com                                             |
  +----------------------------------------------------------------------+
  | The contents of this file are subject to the General Public License  |
  | (GPL) Version 2 (the "License"); you may not use this file except in |
  | compliance with the License. You may obtain a copy of the License at |
  | http://www.opensource.org/licenses/gpl-license.php                   |
  |                                                                      |
  | Software distributed under the License is distributed on an "AS IS"  |
  | basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See  |
  | the License for the specific language governing rights and           |
  | limitations under the License.                                       |
  +----------------------------------------------------------------------+
  | The Original Code is: Elastix Open Source.                           |
  | The Initial Developer of the Original Code is PaloSanto Solutions    |
  +----------------------------------------------------------------------+
  $Id: new_campaign.php $ */

class paloSantoRegisterKey
{
	var $errMsg;
	var $sContenidoTotal;
	private $_sRutaExe;

	function paloSantoRegisterKey($sRutaExe)
	{
		$this->errMsg = '';
		if (!(file_exists($sRutaExe) && is_executable($sRutaExe))) {
		    die('(internal) Invalid registration executable: '.$sRutaExe);
		}
		$this->_sRutaExe = $sRutaExe;
	}
	
	/**
	 * Procedimiento que lista las licencias que tienen el prefijo de nombre de
	 * archivo indicado debajo del directorio de licencias de Asterisk indicado
	 * por la configuración. 
	 *
	 * @param   string  $sPrefijoLicencia   Prefijo esperado de los nombres de
	 *                                      archivo de las licencias.
	 *
	 * @return  mixed   NULL en caso de error, o listado de información de 
	 *                  licencias encontradas.
	 */
	function listarLicencias($sPrefijoLicencia)
	{
		global $config;
        if (!file_exists($config['license_dir'])) { $this->errMsg = '(internal) License directory not found!'; return NULL; }
        if (!is_dir($config['license_dir'])) { $this->errMsg = '(internal) License path not a directory!'; return NULL; }
        if (!is_readable($config['license_dir'])) { $this->errMsg = '(internal) License path not readable!'; return NULL; }

        // Obtener todos los archivos candidatos posibles en el directorio
        $listaArchivos = array();
        $hDir = opendir($config['license_dir']);
        while (($sNombreArchivo = readdir($hDir)) !== false) {
            if (strlen($sNombreArchivo) > strlen($sPrefijoLicencia) + 5 && 
                substr($sNombreArchivo, 0, strlen($sPrefijoLicencia)) == $sPrefijoLicencia &&
                substr($sNombreArchivo, -4) == '.lic') {
                $listaArchivos[] = $sNombreArchivo;
            }
        }
        closedir($hDir);

        // Parsear los datos de cada archivo candidato:
        /*
Key-ID: G729-xxxxx
Product: G.729 Codec
Channels: 5
ExpDate: 2030-05-27
Host-ID: xx:xx:xx:xx:xx:xx
Signature: [datos base64]
        */
        $listaLicencias = array();
        foreach ($listaArchivos as $sNombreArchivo) {
            $sRuta = $config['license_dir'].'/'.$sNombreArchivo;
            $hArchivo = fopen($sRuta, 'rt');
            if ($hArchivo) {
                $infoLicencia = array(
                    'Filename'  =>  $sNombreArchivo,
                );
                while (!feof($hArchivo)) {
                    $sLinea = fgets($hArchivo);
                    $regs = NULL;
                    if (preg_match('/^(.+?):\s*(.+)/', $sLinea, $regs)) {
                        $infoLicencia[$regs[1]] = $regs[2];
                    }
                }
                fclose($hArchivo);
                $listaLicencias[] = $infoLicencia;
            }
        }

		return $listaLicencias;
	}
	
    /**
     * Procedimiento para localizar el archivo de acuerdo de licencia para el 
     * paquete Digium que tiene el prefijo indicado por el parámetro. Se asume
     * que los archivos están en /usr/share/doc/PREFIJO-xxx.xx.xx/LICENSE
     *
     * @param   string  $sPrefijoPaquete    Prefijo a usar para localizar el
     *                                      archivo de licencia
     *
     * @return  mixed   NULL en caso de error, o la cadena de la ruta al archivo.
     */
	function localizarAcuerdoLicencia($sPrefijoPaquete)
	{
	    $sRutaDocs = '/usr/share/doc';
	    $hDir = opendir($sRutaDocs);
	    $listaCandidatos = array();
        while (($sNombreArchivo = readdir($hDir)) !== false) {
            if (is_dir($sRutaDocs.'/'.$sNombreArchivo) && 
                strpos($sNombreArchivo, $sPrefijoPaquete) === 0)
                $listaCandidatos[] = $sRutaDocs.'/'.$sNombreArchivo;            
        }        
        closedir($hDir);
        foreach ($listaCandidatos as $sRutaDir) {
            if (is_readable($sRutaDir.'/LICENSE')) return $sRutaDir.'/LICENSE';
        }
        $this->errMsg = '(internal) License file not found!';
        return NULL;
	}

	function registerDigium($sClave, $paramRegistro)
	{
		$this->sContenidoTotal = '';

		// Verificar si el comando /usr/sbin/register.XXX está registrado
		// correctamente en /etc/sudoers
		$output = $ret = NULL;
		exec("sudo -l", $output, $ret);
		if ($ret != 0) {
			$this->errMsg = '(internal) '.$this->_sRutaExe.' not registered in /etc/sudoers';
			return FALSE;
		}
		$bEncontrado = FALSE;
		foreach ($output as $sLinea) {
		    if (strpos($sLinea, '(root) NOPASSWD: '.$this->_sRutaExe) !== FALSE) $bEncontrado = TRUE;
		}
		if (!$bEncontrado) {
			$this->errMsg = '(internal) '.$this->_sRutaExe.' not registered in /etc/sudoers';
			return FALSE;
		}

		// Verificar si se tienen respuestas a todas las preguntas
		$opcionesMenu = $listaCampos = NULL;
		$this->_verificarMenuDigium($sClave, $listaCampos, $paramRegistro, $opcionesMenu, $listaCampos);
		if (!is_array($opcionesMenu)) {
			return FALSE;
		}

		$bContinuar = TRUE;
		$bResultado = TRUE;
        $descriptores = array(
            0	=>	array('pipe', 'r'),
            1	=>	array('pipe', 'w'),
            2	=>	array('file', '/tmp/register-stderr.txt', 'w'),
        );
        $procPipes = NULL; $cwd = '/';
        $procReg = proc_open('sudo '.$this->_sRutaExe, $descriptores, $procPipes, $cwd);
		stream_set_blocking($procPipes[1], 0);

		// Paso 1: elegir entre Digium y Cepstrel
		if ($bContinuar) {
			$sPregunta = 'Your Choice: ';
			$sContenido = $this->_leerHastaPregunta($procPipes, $sPregunta);
			if ($sContenido === FALSE) {
				$this->errMsg = '(internal) Error while running '.$this->_sRutaExe;
				$bContinuar = FALSE;
				$bResultado = FALSE;
			}
		}
		if ($bContinuar) {
			if (strpos($sContenido, $opcionesMenu[0][1]) === FALSE) {
				$this->errMsg = '(internal) Digium registration menu (step 1) has changed - please report!';
				fwrite($procPipes[0], "0\n");
				$bContinuar = FALSE;
				$bResultado = FALSE;
			}
		}

		if ($bContinuar) {
			// Se elige 1 para Digium
			fwrite($procPipes[0], $opcionesMenu[0][0]."\n");

			// Paso 2: elegir entre productos Digium
			$sPregunta = 'Your Choice: ';
			$sContenido = $this->_leerHastaPregunta($procPipes, $sPregunta);
			if ($sContenido === FALSE) {
				$this->errMsg = '(internal) Could not find end of menu (step 2) - please report!';
				fwrite($procPipes[0], "0\n");
				$bContinuar = FALSE;
				$bResultado = FALSE;
			}
		}
		if ($bContinuar) {
			if (strpos($sContenido, $opcionesMenu[1][1]) === FALSE) {
				$this->errMsg = '(internal) Digium registration menu (step 2) has changed - please report!';
				fwrite($procPipes[0], "0\n");
				$bResultado = FALSE;
				$bContinuar = FALSE;
			}
		}

		if ($bContinuar) {
			fwrite($procPipes[0], $opcionesMenu[1][0]."\n");

			// Paso 3: se introduce clave de registro
			$sPregunta = 'Please enter your Key-ID: ';
			$sContenido = $this->_leerHastaPregunta($procPipes, $sPregunta);
			if ($sContenido === FALSE) {
				$this->errMsg = '(internal) Could not find end of menu (step 3) - please report!';
				fwrite($procPipes[0], "0\n");
				$bResultado = FALSE;
				$bContinuar = FALSE;
			}
		}

		if ($bContinuar) {
			// Se introduce la clave de registro
			fwrite($procPipes[0], $sClave."\n");

			// Paso 4: se confirma ingreso de clave
			$sPregunta = 'Do you want to register this key now(y/n) ';
			$sContenido = $this->_leerHastaPregunta($procPipes, $sPregunta);
			if ($sContenido === FALSE) {
			    if (strpos($this->sContenidoTotal, 'cannot be registered') !== FALSE) {
			        $this->errMsg = 'Invalid license key';
			    } else {
				    $this->errMsg = '(internal) Could not find end of menu (step 4) - please report!';
				}
				fwrite($procPipes[0], "0\n");
				$bResultado = FALSE;
				$bContinuar = FALSE;
			}
		}

		if ($bContinuar) {
			// Se confirma el registro de la clave
			fwrite($procPipes[0], "y\n");
			
			// Paso 5: se acepta el EULA
			$sPregunta = 'Do you accept this licensing agreement (y/n)? ';
			$sContenido = $this->_leerHastaPregunta($procPipes, $sPregunta);
			if ($sContenido === FALSE) {
				$this->errMsg = '(internal) Could not find end of menu (step 5) - please report!';
				fwrite($procPipes[0], "0\n");
				$bResultado = FALSE;
				$bContinuar = FALSE;
			}
		}
		
		if ($bContinuar) {
			// Se confirma el EULA
			fwrite($procPipes[0], "y\n");

			foreach ($listaCampos as $tuplaCampo) {
				$sContenido = $this->_leerHastaPregunta($procPipes, $tuplaCampo[0]);
				if ($sContenido === FALSE) {
					$this->errMsg = '(internal) Digium registration menu (step 6) has changed - please report!';
					fwrite($procPipes[0], "0\n");
					$bResultado = FALSE;
					$bContinuar = FALSE;
				} else {
					$s = isset($paramRegistro[$tuplaCampo[1]]) ? $paramRegistro[$tuplaCampo[1]] : 'x';
					fwrite($procPipes[0], "$s\n");
				}
				if (!$bContinuar) break;
			}
		}

		if ($bContinuar) {
			$sContenido = $this->_leerHastaPregunta($procPipes, 'Wrote license to');
		}

        fclose($procPipes[0]);
        fclose($procPipes[1]);
		// El siguiente código no funciona porque register.g729 corre como root
/*
        $infoReg = proc_get_status($procReg);
        if ($infoReg['running']) {
        	usleep(1000000);
        	posix_kill($infoReg['pid'], 2);
        }
*/        
        $ret = proc_close($procReg);
        return $bResultado;
	}
	
/*
https://register.digium.com/register.php
1|Digium Products
2|Cepstral Products

https://register.digium.com/register.php?module=1&action=listproducts
11|Asterisk Business Edition|ABE
33|Asterisk Business Edition C Expansion|ABEX
45|Asterisk For Smart Cube|ASC
46|Asterisk For Smart Cube Expansion|ASCE
16|G.729 Codec|G729
20|High Performance Echo Can|HPEC
37|Skype For Asterisk|S4A
38|Fax for Asterisk|FAX
40|Free Fax for Asterisk|FFA
49|Vestec Speech Engine|VTE

https://register.digium.com/register.php?module=1&product=16

first_name|First Name: |1||
last_name|Last Name: |1||
company|Company: |0||
address1|Address 1: |1||
address2|Address 2: |0||
city|City: |1||
state|State: |0||
post_code|Postal Code: |0||
country|Country: |1||
phone|Phone: |1||
email|E-Mail: |1||
*/

	private function _verificarMenuDigium($sClave, $listaCampos, $paramRegistro, &$opcionesMenu, &$listaCampos)
	{
		$opcionesMenu = array();
		$sOptDigium = NULL;
		$sOptProducto = NULL;
/*
		$opcionesMenu = array(
			array("1", '1 - Digium Products'),
			array("5", '5 - G.729 Codec'),
		);
		$listaCampos = array(
			array('First Name: ', 'first_name'),
			array('Last Name: ', 'last_name'),
			array('Company: ', 'company'),
			array('Address 1: ', 'address_1'),
			array('Address 2: ', 'address_2'),
			array('City: ', 'city'),
			array('State: ', 'state'),
			array('Postal Code: ', 'postcode'),
			array('Country: ', 'country'),
			array('Phone: ', 'phone'),
			array('E-Mail: ', 'email'),
		);
*/


		$regs = NULL;
		if (!preg_match('/^(\w+)-\w+$/', $sClave, $regs)) {
			$this->errMsg = '(internal) License key not in valid Digium format';
			$opcionesMenu = NULL; return;
		}
		$sTipoProducto = $regs[1];

		if (!function_exists('curl_init')) {
			$this->errMsg = '(internal) No cURL support compiled in!';
			$opcionesMenu = NULL; return;
		}
		$sNombreTemp = tempnam('/tmp', 'verify-digium-');
		$hCurl = curl_init();
		curl_setopt($hCurl, CURLOPT_SSL_VERIFYPEER, FALSE);

		// Leer opciones iniciales para saber cuál dice Digium
		$hArchivo = fopen($sNombreTemp, 'w');
		curl_setopt($hCurl, CURLOPT_FILE, $hArchivo);
		curl_setopt($hCurl, CURLOPT_URL, "https://register.digium.com/register.php");
		$bExito = curl_exec($hCurl);
		fclose($hArchivo);
		if (!$bExito) {
			$this->errMsg = '(internal) Failed to fetch https://register.digium.com/register.php';
			$opcionesMenu = NULL; return;
		}
		$hArchivo = fopen($sNombreTemp, 'r');
		$iLinea = 0;
		while ($campos = fgetcsv($hArchivo, 8192, '|')) {
			$iLinea++;
			if (strpos($campos[1], 'Digium Products') !== FALSE) {
				$opcionesMenu[0] = array($iLinea, $campos[1]);
				$sOptDigium = $campos[0];
			}
		}
		fclose($hArchivo);
		if (count($opcionesMenu) != 1) {
			$this->errMsg = '(internal) Failed to find menu item for Digium Products';
			$opcionesMenu = NULL; return;
		}
		
		// Leer menú de productos para saber cuál tiene el tipo de producto
		$sUrlProductos = "https://register.digium.com/register.php?module=$sOptDigium&action=listproducts";
		$hArchivo = fopen($sNombreTemp, 'w');
		curl_setopt($hCurl, CURLOPT_FILE, $hArchivo);
		curl_setopt($hCurl, CURLOPT_URL, $sUrlProductos);
		$bExito = curl_exec($hCurl);
		fclose($hArchivo);
		if (!$bExito) {
			$this->errMsg = '(internal) Failed to fetch '.$sUrlProductos;
			$opcionesMenu = NULL; return;
		}
		$hArchivo = fopen($sNombreTemp, 'r');
		$iLinea = 0;
		while ($campos = fgetcsv($hArchivo, 8192, '|')) {
			$iLinea++;
			if ($campos[2] == $sTipoProducto) {
				$opcionesMenu[1] = array($iLinea, $campos[1]);
				$sOptProducto = $campos[0];
			}
		}
		fclose($hArchivo);
		if (count($opcionesMenu) != 2) {
			$this->errMsg = '(internal) Failed to find menu item for '.$sTipoProducto;
			$opcionesMenu = NULL; return;
		}
		
		// Leer opciones de registro para saber cuáles campos serán pedidos
		$listaCampos = array();
		$sUrlRegistro = "https://register.digium.com/register.php?module=$sOptDigium&product=$sOptProducto";
		$hArchivo = fopen($sNombreTemp, 'w');
		curl_setopt($hCurl, CURLOPT_FILE, $hArchivo);
		curl_setopt($hCurl, CURLOPT_URL, $sUrlRegistro);
		$bExito = curl_exec($hCurl);
		fclose($hArchivo);
		if (!$bExito) {
			$this->errMsg = '(internal) Failed to fetch '.$sUrlProductos;
			$opcionesMenu = NULL; return;
		}
		$hArchivo = fopen($sNombreTemp, 'r');
		$iLinea = 0;
		while ($campos = fgetcsv($hArchivo, 8192, '|')) {
			$listaCampos[] = array($campos[1], $campos[0]);
			if (!(isset($paramRegistro[$campos[0]]) && trim($paramRegistro[$campos[0]]) != '' || $campos[2] == 0)) {
				$this->errMsg = '(internal) Non-optional parameter '.$campos[0].' blank or not present';
				$opcionesMenu = NULL; return;
			}
		}
		fclose($hArchivo);
		
		@unlink($sNombreTemp);
	}

	private function _leerHastaPregunta($procPipes, $sCadenaPregunta)
	{
		$sContenido = '';

		$bPregunta = FALSE;
		$iTimestampInicio = time();
		while (!$bPregunta) {
			// No esperar más de 10 segundos para leer respuesta
			if (time() - $iTimestampInicio > 10) {
				return FALSE;
			}
		    $listoLeer = array(
		    	$procPipes[1],
		    );
		    $listoEscribir = NULL;
		    $listoErr = NULL;
		    $iNumCambio = stream_select($listoLeer, $listoEscribir, $listoErr, 1);
		    if ($iNumCambio > 0 && in_array($procPipes[1], $listoLeer)) {
		    	$s = stream_get_contents($procPipes[1]);
		    	if ($s == '') {
		    		return FALSE;
		    	}
		    	$sContenido .= $s;
		    	$this->sContenidoTotal .= $s;
		    	if (strpos($sContenido, $sCadenaPregunta) !== FALSE)
		    		$bPregunta = TRUE;
			}
		}
		return $sContenido;
	}
}

?>
